@@ -88,10 +88,12 @@ def test_del(connector, key, protocol, loop) -> None:
         gc.collect()
 
     connector._release.assert_called_with(key, protocol, should_close=True)
-    msg = {'client_connection': mock.ANY,  # conn was deleted
-           'message': 'Unclosed connection'}
+    msg = {
+        "client_connection": mock.ANY,  # conn was deleted
+        "message": "Unclosed connection",
+    }
     if loop.get_debug():
-        msg['source_traceback'] = mock.ANY
+        msg["source_traceback"] = mock.ANY
     loop.call_exception_handler.assert_called_with(msg)
 
 
